==> em_celery-0.1.18-py2.py3-none-any.whl: em_celery/tools/ebay_products_fetch_seed_feeder.py <==
# -*- coding: utf-8 -*-

import os
import sys
import datetime
import time
import json
try:
  from urllib.parse import urlparse
  from urllib.parse import parse_qs
except ImportError:
  from urlparse import urlparse
  from urlparse import parse_qs

import click
import redis
import dateparser

from em_celery import logger, get_product_service


@click.command('Send ebay products crawl seed urls')
@click.option('-u', '--redis_uri', required=True, type=str, help='Redis URI.')
@click.option('-q', '--qps', type=float, default=20, help='Seeds add rate in seconds.')
@click.option('-t', '--ttl', type=int, default=3, help='Product information alive days, default is 3.')
@click.option('-k', '--start_urls_key', required=True, type=str, default='ebay_product:start_urls', help='Start URLs key.')
@click.argument('products_path')
def feed_seeds(products_path, redis_uri, qps=20, start_urls_key='ebay_product:start_urls', ttl=3):
  products_path = os.path.abspath(os.path.expanduser(products_path))
  if not os.path.isfile(products_path):
    print('Could not find products path - %s' % products_path)
    sys.exit(1)

  feeder = EbayProductsFetchSeedFeeder(products_path, redis_uri, start_urls_key, qps, ttl)
  feeder.run()


class EbayProductsFetchSeedFeeder():
  def __init__(self, products_path, redis_uri, start_urls_key, qps, ttl=3):
    self.products_path = products_path
    self.r = redis.Redis.from_url(redis_uri)
    self.start_urls_key = start_urls_key
    self.qps = qps
    self.last_send_time = None
    self.product_service = get_product_service()
    self.indice_name = 'ebay_us_products'
    today = datetime.date.today()
    self.expire_date = today - datetime.timedelta(days=ttl) 

  def run(self):
    product_urls = []
    batch_size = 500
    try:
      with open(self.products_path, encoding='utf-8', errors='ignore') as fh:
        for line in fh:
          s = line.strip().split("\t")
          if not s:
            continue

          url = None
          product_url = s.pop()
          if product_url.find('/p/') == -1:
            url = product_url
          else:
            parsed_url = urlparse(product_url)
            query_params = parse_qs(parsed_url.query)
            pid = query_params.get('iid', [''])[0]
            url = '{}://{}/itm/{}'.format(parsed_url.scheme, parsed_url.netloc, pid)

          if not url:
            continue

          product_urls.append(url)

          if len(product_urls) < batch_size:
            continue

          self.process_products(product_urls)
          product_urls = []
    except (KeyboardInterrupt, SystemExit):
      print('Exiting, please wait...')
    finally:
      if product_urls:
        self.process_products(product_urls)
        product_urls = []

  def process_products(self, product_urls):
    pids = dict()
    for product_url in product_urls:
      pid = self.extract_product_id(product_url)
      if not pid:
        continue

      pids[pid] = product_url

    products = False
    max_retries = 3
    while max_retries > 0:
      try:
        products = self.product_service.search_products(self.indice_name, list(pids.keys()))
        if not isinstance(products, dict):
          logger.info('[ProductService] Temporary unavailable! Wait for 10 seconds to retry.')
          max_retries -= 1
          time.sleep(10)
          continue

        break
      except Exception as e:
        logger.exception(e)
        max_retries -= 1
        time.sleep(3)

    if not products:
      return

    for pid, product in products.items():
      exist = product.get('existence', True)
      if not exist:
        continue

      update_date_s = product.get('date', product.get('time', None))
      if not update_date_s:
        continue

      try:
        update_date = dateparser.parse(update_date_s).date()
        # Not expired, no need to update
        if update_date > self.expire_date:
          pids.pop(pid, None)
      except:
        continue

    urls = list(pids.values())
    for product_url in urls:
      if self.last_send_time:
        wait_time = 1 / self.qps - (time.time() - self.last_send_time)
        if wait_time > 0:
          print("Waiting %.3fs to send next message" % wait_time)
          time.sleep(wait_time)

      self.last_send_time = time.time()

      self.r.lpush(self.start_urls_key, json.dumps({'url': product_url}))
      logger.debug('Added %s', product_url)

  def extract_product_id(self, product_url):
    try:
      return urlparse(product_url).path.split('/').pop()
    except Exception as e:
      self.logger.exception(e)


if __name__ == "__main__":
  feed_seeds()

==> em_celery-0.1.38-py2.py3-none-any.whl: em_celery/tools/ebay_products_fetch_seed_feeder.py <==
# -*- coding: utf-8 -*-

import os
import sys
import datetime
import time
import json
try:
  from urllib.parse import urlparse
  from urllib.parse import parse_qs
except ImportError:
  from urlparse import urlparse
  from urlparse import parse_qs

import click
import redis
import dateparser

from em_celery import logger, get_product_service


@click.command('Send ebay products crawl seed urls')
@click.option('-u', '--redis_uri', required=True, type=str, help='Redis URI.')
@click.option('-q', '--qps', type=float, default=20, help='Seeds add rate in seconds.')
@click.option('-t', '--ttl', type=int, default=3, help='Product information alive days, default is 3.')
@click.option('-k', '--start_urls_key', required=True, type=str, default='ebay_product:start_urls', help='Start URLs key.')
@click.argument('products_path')
def feed_seeds(products_path, redis_uri, qps=20, start_urls_key='ebay_product:start_urls', ttl=3):
  products_path = os.path.abspath(os.path.expanduser(products_path))
  if not os.path.isfile(products_path):
    print('Could not find products path - %s' % products_path)
    sys.exit(1)

  feeder = EbayProductsFetchSeedFeeder(products_path, redis_uri, start_urls_key, qps, ttl)
  feeder.run()


class EbayProductsFetchSeedFeeder():
  def __init__(self, products_path, redis_uri, start_urls_key, qps, ttl=3):
    self.products_path = products_path
    self.r = redis.Redis.from_url(redis_uri)
    self.start_urls_key = start_urls_key
    self.qps = qps
    self.last_send_time = None
    self.product_service = get_product_service()
    self.indice_name = 'ebay_us_products'
    today = datetime.date.today()
    self.expire_date = today - datetime.timedelta(days=ttl) 

  def run(self):
    product_urls = []
    batch_size = 500
    try:
      with open(self.products_path, encoding='utf-8', errors='ignore') as fh:
        for line in fh:
          s = line.strip().split("\t")
          if not s:
            continue

          url = None
          product_url = s.pop()
          if product_url.find('/p/') == -1:
            url = product_url
          # else:
          #   parsed_url = urlparse(product_url)
          #   query_params = parse_qs(parsed_url.query)
          #   pid = query_params.get('iid', [''])[0]
          #   url = '{}://{}/itm/{}'.format(parsed_url.scheme, parsed_url.netloc, pid)

          if not url:
            continue

          product_urls.append(url)

          if len(product_urls) < batch_size:
            continue

          self.process_products(product_urls)
          product_urls = []
    except (KeyboardInterrupt, SystemExit):
      print('Exiting, please wait...')
    finally:
      if product_urls:
        self.process_products(product_urls)
        product_urls = []

  def process_products(self, product_urls):
    pids = dict()
    for product_url in product_urls:
      pid = self.extract_product_id(product_url)
      if not pid:
        continue

      pids[pid] = product_url

    products = False
    max_retries = 3
    while max_retries > 0:
      try:
        products = self.product_service.search_products(self.indice_name, list(pids.keys()))
        if not isinstance(products, dict):
          logger.info('[ProductService] Temporary unavailable! Wait for 10 seconds to retry.')
          max_retries -= 1
          time.sleep(10)
          continue

        break
      except Exception as e:
        logger.exception(e)
        max_retries -= 1
        time.sleep(3)

    if not products:
      return

    for pid, product in products.items():
      exist = product.get('existence', True)
      if not exist:
        continue

      update_date_s = product.get('date', product.get('time', None))
      if not update_date_s:
        continue

      try:
        update_date = dateparser.parse(update_date_s).date()
        # Not expired, no need to update
        if update_date > self.expire_date:
          pids.pop(pid, None)
      except:
        continue

    urls = list(pids.values())
    for product_url in urls:
      if self.last_send_time:
        wait_time = 1 / self.qps - (time.time() - self.last_send_time)
        if wait_time > 0:
          print("Waiting %.3fs to send next message" % wait_time)
          time.sleep(wait_time)

      self.last_send_time = time.time()

      self.r.lpush(self.start_urls_key, json.dumps({'url': product_url}))
      logger.debug('Added %s', product_url)

  def extract_product_id(self, product_url):
    try:
      return urlparse(product_url).path.split('/').pop()
    except Exception as e:
      self.logger.exception(e)


if __name__ == "__main__":
  feed_seeds()
